import pandas as pd
import yaml
from sqlalchemy import create_engine,text

def testconnection(file = None):
    databasedict = {}
    databaseenginedict = {}
    with open(file) as f:
        databasedata = yaml.load(f, Loader=yaml.FullLoader)

    for databasekey in databasedata.keys():
        print( "测试 %s 连接情况"%databasekey )
        database = databasedata[databasekey]
        try:
            engine = create_engine(database,pool_timeout=10)
            try:
                with engine.connect() as connection:
                    result = connection.execute(text("SELECT 1"))
                    print("连接成功:", result.fetchone())
                    databasedict[databasekey] =  database
                    databaseenginedict['engine_%s'%databasekey] =  engine
            except Exception as e:
                print("连接失败:", e)
        except:
            databasedict[databasekey] = database
    assert len(databasedict)>0, "没有有效数据库连接"
    return databasedict,databaseenginedict


class database(object):
    def __init__(self,file = None):
        self.databasedict,self.databaseenginedict = testconnection(file=file)
        self.__dict__.update(self.databasedict)
        self.__dict__.update(self.databaseenginedict)











# -*- coding: <encoding name> -*
import reprlib

try:
    from collections.abc import Iterable
except ImportError:
    from collections import Iterable


import pandas as pd
from sqlalchemy import MetaData, create_engine,cast,NVARCHAR
from sqlalchemy.engine import reflection
from sqlalchemy.exc import InvalidRequestError
from sqlalchemy.ext.automap import automap_base
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm.attributes import InstrumentedAttribute
from sqlalchemy.orm import Session,Query


class HQuery(Query):
    def to_df(self, **kwargs):
        """[pandas.read_sql]

        Arguments:
            Query {[type]} -- [description]

        Returns:
            [pd.DataFrame or generate] -- [description]
        """
        return pd.read_sql(sql=self.statement, con=self.session.bind, **kwargs)


class HSession(Session):
    def __init__(self,
                 bind=None,
                 autoflush=True,
                 future=True,
                 expire_on_commit=True,
                 autocommit=False,
                 twophase=False,
                 binds=None,
                 enable_baked_queries=True,
                 info=None,
                 query_cls=HQuery):
        super().__init__(
            bind=bind,
            autoflush=autoflush,
            future=future,
            expire_on_commit=expire_on_commit,
            autocommit=autocommit,
            twophase=twophase,
            binds=binds,
            enable_baked_queries=enable_baked_queries,
            info=info,
            query_cls=query_cls)


class DataBase:
    """class for easy to use orm of your database/

    Raises:
        ValueError -- [bind must be sth like sqlalchemy's engine]

    Returns:
        [instance of DataBase] -- []
    """

    def __init__(self, bind, schema=None):
        if bind is None:
            return
        if schema == 'None':
            schema = None
        if isinstance(bind, str):
            bind = create_engine(bind)
        self._bind = bind
        self._schema = schema
        self._metadata = MetaData(self._schema)
        self.Base = declarative_base(metadata=self._metadata)
        self._session = HSession(self._bind)
        self.tables = self._get_tables(self._bind, schema)
        for table in self.tables:
            setattr(self, table, 'None')
            # 可能有大小写问题
            setattr(self, table.lower(), 'None')
            setattr(self, table.upper(), 'None')

    def inject(self, bind, schema=None):
        self.__init__(bind, schema)
        return

    def __getattribute__(self, table_name):
        if object.__getattribute__(self, table_name) == 'None':
            try:
                setattr(self, table_name, self._reflect_table(table_name))
            except InvalidRequestError:
                try:
                    setattr(self, table_name,
                            self._reflect_table(table_name.lower()))
                except InvalidRequestError:
                    setattr(self, table_name,
                            self._reflect_table(table_name.upper()))
        return object.__getattribute__(self, table_name)

    def __getattr__(self, name):
        raise AttributeError(
            f'<{name}> is not the right table name, such as {reprlib.repr(self.tables)}.'
        )

    def __repr__(self):
        return str(self._bind).replace(
            type(self._bind).__name__,
            type(self).__name__)

    @property
    def bind(self):
        return self._bind

    @property
    def schema(self):
        return self._schema

    def _reflect_table(self, table_name):

        # self._metadata.reflect(bind = self.bind ,only=[table_name])
        self._metadata.reflect(bind=self._bind,schema=self._schema,only=[table_name])
        Base = automap_base(metadata=self._metadata)
        Base.prepare()
        try:
            table = Base.classes[table_name]
            column_list = tuple(table.__dict__.keys())
            # 对列名大小写进行处理
            for column in column_list:
                c = getattr(table, column)
                if isinstance(c, InstrumentedAttribute):
                    setattr(table, column.upper(), c)
                    setattr(table, column.lower(), c)
            return table
        except KeyError:
            raise ValueError(f"There must be a primary key in {table_name}! \
or <{table_name}> is not the right table name, such as {reprlib.repr(self.tables)}."
                             )

    def reflect(self):
        self._metadata.reflect(bind = self.bind )
        Base = automap_base(metadata=self._metadata)
        Base.prepare()
        for k, v in Base.classes.items():
            setattr(self, k, v)
            setattr(self, k.lower(), v)
            setattr(self, k.upper(), v)

    @staticmethod
    def _get_tables(bind, schema):
        insp = reflection.Inspector.from_engine(bind=bind)
        tables = insp.get_table_names(schema=schema)
        return tables

    def query(self, *columns):
        """[session.query]

        Returns:
            [Query] -- [sqlalchemy Query cls]
        """

        return self._session.query(*columns)

    def update(self, t_obj):
        """[update table]

        Arguments:
            t_obj {[objs of DeclarativeMeta]} -- [update the table]
        """

        if isinstance(t_obj, Iterable):
            self._session.add_all(t_obj)
        else:
            self._session.add(t_obj)

    def insert(self, table, insert_obj, ignore=True, index=None):
        """[insert bulk data]

        Arguments:
            table {[DeclarativeMeta cls]} -- [reflection of table]
            insert_obj {[pd.DataFrame or list of dicts]} -- [insert_obj]
            ignore {[bool]} -- [whether or not ignore duplicate insert]
            index {[str]} -- [the index you want to ignore]

        Keyword Arguments:
            ignore {bool} -- [wether ignore exception or not] (default: {True})

        Raises:
            ValueError -- [f"The {reprlib.repr(insert_obj)} must be list of dicts type!"]

        Returns:
            [type] -- [description]
        """

        if isinstance(insert_obj, pd.DataFrame):
            if insert_obj.empty:
                raise ValueError('The input DataFrame is empty, please check!')
            insert_obj = insert_obj.to_dict('records')
        elif not isinstance(insert_obj, list):
            raise ValueError(
                f"The {reprlib.repr(insert_obj)} must be list of dicts type!")
        if self._bind.dialect.name == 'mysql':
            ignore_str = 'IGNORE' if ignore else ''
        elif self._bind.dialect.name == 'sqlite':
            ignore_str = 'OR REPLACE' if ignore else ''
        elif self._bind.dialect.name == 'oracle':
            if ignore and index is None:
                for i in table.__table__.indexes:
                    if i.unique:
                        index = i.name
                        break
                if index is None:
                    index = list(table.__table__.constraints)[0].name
            ignore_str = f'/*+ IGNORE_ROW_ON_DUPKEY_INDEX ({table.__table__.name}, {index}) */' if ignore else ''
        elif self._bind.dialect.name == 'mssql':
            ignore_str = ""
            if ignore:
                raise ValueError(
                    "You can set 'IGNORE_DUP_KEY = ON' while create the table and set the paramenter 'ignore=False'.")
        return self._session.execute(
            table.__table__.insert().prefix_with(ignore_str).values(insert_obj))


    def delete(self, t_obj):
        return self._session.delete(t_obj)

    def close(self):
        return self._session.close()

    def merge(self, t_obj):
        """[replace]

        Arguments:
            t_obj {[objs of DeclarativeMeta]} -- [replace the table]
        """

        if isinstance(t_obj, Iterable):
            for t in t_obj:
                self._session.merge(t)
            return
        else:
            return self._session.merge(t_obj)


    def commit(self):
        return self._session.commit()

    def flush(self, objects=None):
        return self._session.flush(objects=objects)

    def rollback(self):
        return self._session.rollback()

    def refresh(self):
        return self.__init__(self._bind, self._schema)

    def create_all(self, tables=None, checkfirst=True):
        self._metadata.create_all(
            tables=tables, checkfirst=checkfirst)
        self.refresh()
        return


def get_db(engine, schem):
    try:
        return DataBase(engine, schem)
    except:
        return DataBase(None)

"""
连接wind
bind = 'mssql+pymssql://xxx:xxxx@xxxx:xxx/nWind?charset=utf8'
d_t = HDB.ASHAREDESCRIPTION
stk_desc = HDB.query(d_t.S_INFO_WINDCODE.label('sid'),
                         cast(d_t.S_INFO_NAME ,NVARCHAR(100)).label('name')).filter(d_t.S_INFO_NAME == '塔牌集团').to_df()

连接朝阳永续
bind = 'mssql+pymssql://xxx:xxxx@xxxx:xxx/gogoal2?charset=utf8'
HDB = get_db(bind, schema)
d_t = HDB.con_forecast_stk
stk_desc = HDB.query(d_t .stock_code.label('sid'),
                        cast( d_t .stock_name,NVARCHAR(100) ).label('name')).filter( d_t .con_date>'2024-05-01').to_df()
"""

if __name__ == '__main__':
    pass












from functools import lru_cache
from sqlalchemy.sql import func
from sqlalchemy.dialects.mssql import NVARCHAR
from sqlalchemy import cast
import time
import datetime
import pandas as pd

# "AIndexHS300FreeWeight".upper()
# db = database(file=r'D:\FACTOR\sqldatabase.yaml')
# sourcedb = get_db(db.sourcedatabase_wind, schem='dbo')
# cf = corefunc(sourcedb = sourcedb )

class corefunc:

    @staticmethod
    def get_index_eod(indexs = ['000300.SH','000905.SH','000852.SH','932000.CSI','000985.CSI'], sourcedb = None, begin_dt= '20000101' , end_dt = None  ):
        if end_dt is None:
            end_dt = datetime.datetime.today().strftime('%Y%m%d')
        indexeod = sourcedb.AINDEXEODPRICES
        df = sourcedb.query(indexeod.S_INFO_WINDCODE.label('sid'),
                            indexeod.TRADE_DT.label('dt'),
                            (indexeod.S_DQ_CLOSE).label('close'),
                            ).filter(
            indexeod.TRADE_DT >= begin_dt,
            indexeod.TRADE_DT <= end_dt, indexeod.S_INFO_WINDCODE.in_(indexs)).order_by(indexeod.TRADE_DT, indexeod.S_INFO_WINDCODE).to_df()
        return df

    #返回指数成分股权重
    @staticmethod
    def get_index_weight(indexs = ['000300.SH','000905.SH','000852.SH','932000.CSI','000985.CSI'], sourcedb = None):
        subsheet = sourcedb.AINDEXHS300FREEWEIGHT
        df = sourcedb.query(subsheet.S_INFO_WINDCODE.label('indexcode'), subsheet.S_CON_WINDCODE.label('sid'),
                            subsheet.TRADE_DT, subsheet.I_WEIGHT).filter(
            subsheet.S_INFO_WINDCODE.in_(indexs)).to_df()
        return df

    #返回指数成分股
    @staticmethod
    def get_index_members(indexs = ['000300.SH','000905.SH','000852.SH','932000.CSI','000985.CSI'], sourcedb=None):
        '''
        Args:
            indexs: 指数列表
            sourcedb:
        Returns: 返回指数列表构成的
        '''
        subsheet = sourcedb.AINDEXMEMBERS
        df = sourcedb.query(subsheet.S_INFO_WINDCODE.label('indexcode'),subsheet.S_CON_WINDCODE.label('sid'),subsheet.S_CON_INDATE,subsheet.S_CON_OUTDATE ).filter( subsheet.S_INFO_WINDCODE.in_(indexs)).to_df()
        return df

    # 返回交易日序列
    @staticmethod
    def get_trade_dt(dts=None, dte=None, sourcedb=None, offset='first', period='M'):
        '''

        Args:
            dts: 开始日期 默认为 19910101
            dte: 结束日期 默认为 当前日期
            sourcedb: 数据源
            offset: first 取左，last 取右 默认为 取左
            period: 取数周期

        Returns:

        '''
        if dts is None:
            dts = '19910101'
        if dte is None:
            dte = datetime.datetime.now().strftime("%Y%m%d")
        subsheet = sourcedb.ASHARECALENDAR
        df = sourcedb.query(subsheet.TRADE_DAYS).filter(subsheet.TRADE_DAYS >= dts, subsheet.TRADE_DAYS <= dte,
                                                        subsheet.S_INFO_EXCHMARKET == 'SSE').order_by(
            subsheet.TRADE_DAYS).to_df()
        df['dt'] = pd.to_datetime(df.TRADE_DAYS)
        df.set_index('dt', inplace=True)
        dfresample = df.resample(period)
        tddt = dfresample.agg(offset)
        return tddt

    @staticmethod
    def get_industry_all(level=1, sourcedb=None):
        '''
        Args:
            level: 中信行业分类
            sourcedb: 数据源
        Returns:
            中信行业分类情况
        '''
        if level == 1:
            return corefunc.get_citic_indu_levelone(sourcedb=sourcedb)
        elif level == 2:
            return corefunc.get_citic_indu_leveltwo(sourcedb=sourcedb)
        else:
            return corefunc.get_citic_indu_levelthree(sourcedb=sourcedb)

    @staticmethod
    @lru_cache(maxsize=512)
    def get_indu_bydt(trade_dt, sid=None, level=1, sourcedb=None):
        """
        Arguments:
            trade_dt '%Y%m%d' 格式
            sid {[str or iterable]} -- [sids of stocks] (default: {None})
            level {int} -- [level of zx industry] (default: {1})
            adjust {bool} -- [由于中信变更行业分类，是否调整兼容之前的代码] (default: {True})
        Returns:
            [pd.DataFrame] -- [sid: ind]
        """
        df = corefunc.get_industry_all(level=level, sourcedb=sourcedb)
        if sid is not None:
            sid = {sid} if isinstance(sid, str) else set(sid)
            df = df[df['sid'].isin(sid)]
        df = df.loc[(df['S_CON_INDATE'] <= trade_dt) & (
                (df['S_CON_OUTDATE'] >= trade_dt) | (df['S_CON_OUTDATE'].isnull()))].copy()
        return df.set_index('sid')

    @staticmethod
    def get_eodindicator(dts=None, dte=None, stocklist=None, sourcedb=None):
        '''

        Args:
            dts: 开始时间
            dte: 结束时间
            stocklist: 股票池
            sourcedb: 数据源

        Returns:

        '''
        mv_table = sourcedb.ASHAREEODDERIVATIVEINDICATOR

        if dts is None:
            dts = '20030101'
        if dte is None:
            dte = time.strftime('%Y%m%d', time.localtime())

        if stocklist is None:
            df = sourcedb.query(mv_table.S_INFO_WINDCODE.label('sid'), mv_table.TRADE_DT.label('dt'),
                                mv_table.S_VAL_MV.label('val_mv'), mv_table.S_DQ_TURN.label('tvratio'), mv_table.S_DQ_MV.label('dqmv'),
                                mv_table.UP_DOWN_LIMIT_STATUS.label('updownlimit')).filter(
                mv_table.TRADE_DT >= dts, mv_table.TRADE_DT <= dte, mv_table.S_DQ_TURN != None).order_by(
                mv_table.TRADE_DT, mv_table.S_INFO_WINDCODE).to_df()
        else:
            df = sourcedb.query(mv_table.S_INFO_WINDCODE.label('sid'), mv_table.TRADE_DT.label('dt'),
                                mv_table.S_VAL_MV.label('val_mv'), mv_table.S_DQ_TURN.label('tvratio'), mv_table.S_DQ_MV.label('dqmv'),
                                mv_table.UP_DOWN_LIMIT_STATUS.label('updownlimit')).filter(
                mv_table.TRADE_DT >= dts, mv_table.TRADE_DT <= dte, mv_table.S_INFO_WINDCODE.in_(stocklist),
                mv_table.S_DQ_TURN != None).order_by(
                mv_table.TRADE_DT, mv_table.S_INFO_WINDCODE).to_df()

        return df

    @staticmethod
    @lru_cache(maxsize=512)
    def get_mv_bydt(trade_dt, sourcedb=None):
        mv_table = sourcedb.ASHAREEODDERIVATIVEINDICATOR
        df = sourcedb.query(mv_table.S_INFO_WINDCODE.label('sid'),
                            mv_table.S_VAL_MV.label('mv_suffix')).filter(
            mv_table.TRADE_DT == trade_dt).to_df()
        df.set_index('sid', inplace=True)
        return df

    # 返回估值
    @staticmethod
    def get_stockvaluettm(dts=None, dte=None, stocklist=None, sourcedb=None):
        '''

        Args:
            dts: 开始取数时间，默认是20000101
            dte: 取数结束时间，默认是今天
            stocklist: 如果没有股票序列，那么取全部
            sourcedb: 数据源

        Returns:
            S_INFO_WINDCODE,TRADE_DT,S_VAL_MV,S_VAL_PE_TTM,S_VAL_PCF_OCFTTM,S_VAL_PB_NEW
        '''
        subsheet = sourcedb.ASHAREEODDERIVATIVEINDICATOR
        if dts is None:
            dts = '20030101'
        if dte is None:
            dte = time.strftime('%Y%m%d', time.localtime())
        if stocklist is None:
            df = sourcedb.query(subsheet.S_INFO_WINDCODE, subsheet.TRADE_DT, subsheet.S_VAL_MV, subsheet.S_VAL_PE_TTM,
                                subsheet.S_VAL_PCF_OCFTTM, subsheet.S_VAL_PB_NEW).filter(
                subsheet.TRADE_DT >= dts, subsheet.TRADE_DT <= dte).order_by(
                subsheet.TRADE_DT, subsheet.S_INFO_WINDCODE).to_df()
        else:
            df = sourcedb.query(subsheet.S_INFO_WINDCODE, subsheet.TRADE_DT, subsheet.S_VAL_MV, subsheet.S_VAL_PE_TTM,
                                subsheet.S_VAL_PCF_OCFTTM, subsheet.S_VAL_PB_NEW).filter(
                subsheet.TRADE_DT >= dts, subsheet.TRADE_DT <= dte, subsheet.S_INFO_WINDCODE.in_(stocklist)).order_by(
                subsheet.TRADE_DT, subsheet.S_INFO_WINDCODE).to_df()
        return df

    # 返回股价
    @staticmethod
    def get_stockeodprice(dts=None, dte=None, stocklist=None, sourcedb=None):
        '''
        Args:
            dts: 开始取数时间，默认是20000101
            dte: 取数结束时间，默认是今天
            stocklist: 如果没有股票序列，那么取全部
            sourcedb: 数据源

        Returns:
            返回股票名，日期，开盘价，收盘价，复权开盘价，复权收盘价，当日均价，涨停价，跌停价，成交量
        '''
        subsheet = sourcedb.ASHAREEODPRICES
        if dts is None:
            dts = '20030101'
        if dte is None:
            dte = time.strftime('%Y%m%d', time.localtime())
        if stocklist is None:
            df = sourcedb.query(subsheet.S_INFO_WINDCODE.label('sid'), subsheet.TRADE_DT.label('dt'),
                                subsheet.S_DQ_OPEN,subsheet.S_DQ_HIGH,subsheet.S_DQ_LOW, subsheet.S_DQ_CLOSE,
                                subsheet.S_DQ_ADJOPEN,subsheet.S_DQ_ADJHIGH, subsheet.S_DQ_ADJLOW,  subsheet.S_DQ_ADJCLOSE,
                                subsheet.S_DQ_AVGPRICE,subsheet.S_DQ_AMOUNT,subsheet.S_DQ_ADJFACTOR,
                                subsheet.S_DQ_LIMIT, subsheet.S_DQ_STOPPING, subsheet.S_DQ_VOLUME, subsheet.S_DQ_PCTCHANGE).filter(
                subsheet.TRADE_DT >= dts, subsheet.TRADE_DT <= dte).order_by(
                subsheet.TRADE_DT, subsheet.S_INFO_WINDCODE).to_df()
        else:
            df = sourcedb.query(subsheet.S_INFO_WINDCODE.label('sid'), subsheet.TRADE_DT.label('dt'),
                                subsheet.S_DQ_OPEN,subsheet.S_DQ_HIGH,subsheet.S_DQ_LOW, subsheet.S_DQ_CLOSE,
                                subsheet.S_DQ_ADJOPEN,subsheet.S_DQ_ADJHIGH, subsheet.S_DQ_ADJLOW,  subsheet.S_DQ_ADJCLOSE,
                                subsheet.S_DQ_AVGPRICE,subsheet.S_DQ_AMOUNT,subsheet.S_DQ_ADJFACTOR,
                                subsheet.S_DQ_LIMIT, subsheet.S_DQ_STOPPING, subsheet.S_DQ_VOLUME, subsheet.S_DQ_PCTCHANGE).filter(
                subsheet.TRADE_DT >= dts, subsheet.TRADE_DT <= dte, subsheet.S_INFO_WINDCODE.in_(stocklist)).order_by(
                subsheet.TRADE_DT, subsheet.S_INFO_WINDCODE).to_df()
        return df


    # 返回股价
    @staticmethod
    def get_eodprice(dts=None, dte=None, stocklist=None, sourcedb=None):
        '''
        Args:
            dts: 开始取数时间，默认是20000101
            dte: 取数结束时间，默认是今天
            stocklist: 如果没有股票序列，那么取全部
            sourcedb: 数据源

        Returns:
            返回股票名，日期，开盘价，收盘价，复权开盘价，复权收盘价，当日均价，涨停价，跌停价，成交量
        '''
        subsheet = sourcedb.ASHAREEODPRICES
        if dts is None:
            dts = '20030101'
        if dte is None:
            dte = time.strftime('%Y%m%d', time.localtime())
        if stocklist is None:
            df = sourcedb.query(subsheet.S_INFO_WINDCODE.label('sid'), subsheet.TRADE_DT.label('dt'),
                                subsheet.S_DQ_OPEN, subsheet.S_DQ_HIGH, subsheet.S_DQ_LOW, subsheet.S_DQ_CLOSE,
                                subsheet.S_DQ_ADJOPEN, subsheet.S_DQ_ADJHIGH, subsheet.S_DQ_ADJLOW,
                                subsheet.S_DQ_ADJCLOSE,
                                subsheet.S_DQ_AVGPRICE, subsheet.S_DQ_AMOUNT, subsheet.S_DQ_ADJFACTOR,
                                subsheet.S_DQ_LIMIT, subsheet.S_DQ_STOPPING, subsheet.S_DQ_VOLUME).filter(
                subsheet.TRADE_DT >= dts, subsheet.TRADE_DT <= dte).order_by(
                subsheet.TRADE_DT, subsheet.S_INFO_WINDCODE).to_df()
        else:
            df = sourcedb.query(subsheet.S_INFO_WINDCODE.label('sid'), subsheet.TRADE_DT.label('dt'),
                                subsheet.S_DQ_OPEN, subsheet.S_DQ_HIGH, subsheet.S_DQ_LOW, subsheet.S_DQ_CLOSE,
                                subsheet.S_DQ_ADJOPEN, subsheet.S_DQ_ADJHIGH, subsheet.S_DQ_ADJLOW,
                                subsheet.S_DQ_ADJCLOSE,
                                subsheet.S_DQ_AVGPRICE, subsheet.S_DQ_AMOUNT, subsheet.S_DQ_ADJFACTOR,
                                subsheet.S_DQ_LIMIT, subsheet.S_DQ_STOPPING, subsheet.S_DQ_VOLUME).filter(
                subsheet.TRADE_DT >= dts, subsheet.TRADE_DT <= dte, subsheet.S_INFO_WINDCODE.in_(stocklist)).order_by(
                subsheet.TRADE_DT, subsheet.S_INFO_WINDCODE).to_df()
        return df


    # 返回中信一级行业
    @staticmethod
    @lru_cache()
    def get_citic_indu_levelone(sourcedb=None):
        AINDEXMEMBERSCITICS = sourcedb.AINDEXMEMBERSCITICS
        B = sourcedb.AINDEXDESCRIPTION
        C = sourcedb.ASHAREDESCRIPTION

        df = sourcedb.query(
            AINDEXMEMBERSCITICS.S_INFO_WINDCODE.label('INDEX_INFO_WINDCODE'),
            AINDEXMEMBERSCITICS.S_CON_WINDCODE.label('sid'),
            AINDEXMEMBERSCITICS.S_CON_INDATE,
            AINDEXMEMBERSCITICS.S_CON_OUTDATE,
            cast(B.S_INFO_NAME, NVARCHAR).label('INDEX_INFO_NAME'),
            cast(C.S_INFO_NAME, NVARCHAR).label('S_INFO_NAME'),
            C.S_INFO_DELISTDATE,
            C.S_INFO_LISTDATE
        ).outerjoin(B, AINDEXMEMBERSCITICS.S_INFO_WINDCODE == B.S_INFO_WINDCODE).outerjoin(C,
                                                                                           AINDEXMEMBERSCITICS.S_CON_WINDCODE == C.S_INFO_WINDCODE).to_df()
        return df

    # 返回中信二级行业
    @staticmethod
    @lru_cache()
    def get_citic_indu_leveltwo(sourcedb=None):
        AINDEXMEMBERSCITICS = sourcedb.AINDEXMEMBERSCITICS2
        B = sourcedb.AINDEXDESCRIPTION
        C = sourcedb.ASHAREDESCRIPTION

        df = sourcedb.query(
            AINDEXMEMBERSCITICS.S_INFO_WINDCODE.label('INDEX_INFO_WINDCODE'),
            AINDEXMEMBERSCITICS.S_CON_WINDCODE.label('sid'),
            AINDEXMEMBERSCITICS.S_CON_INDATE,
            AINDEXMEMBERSCITICS.S_CON_OUTDATE,
            cast(B.S_INFO_NAME, NVARCHAR).label('INDEX_INFO_NAME'),
            cast(C.S_INFO_NAME, NVARCHAR).label('S_INFO_NAME'),
            C.S_INFO_DELISTDATE,
            C.S_INFO_LISTDATE
        ).outerjoin(B, AINDEXMEMBERSCITICS.S_INFO_WINDCODE == B.S_INFO_WINDCODE).outerjoin(C,
                                                                                           AINDEXMEMBERSCITICS.S_CON_WINDCODE == C.S_INFO_WINDCODE).to_df()
        return df

    # 返回中信三级行业
    @staticmethod
    @lru_cache()
    def get_citic_indu_levelthree(sourcedb=None):
        AINDEXMEMBERSCITICS = sourcedb.AINDEXMEMBERSCITICS3
        B = sourcedb.AINDEXDESCRIPTION
        C = sourcedb.ASHAREDESCRIPTION

        df = sourcedb.query(
            AINDEXMEMBERSCITICS.S_INFO_WINDCODE.label('INDEX_INFO_WINDCODE'),
            AINDEXMEMBERSCITICS.S_CON_WINDCODE.label('sid'),
            AINDEXMEMBERSCITICS.S_CON_INDATE,
            AINDEXMEMBERSCITICS.S_CON_OUTDATE,
            cast(B.S_INFO_NAME, NVARCHAR).label('INDEX_INFO_NAME'),
            cast(C.S_INFO_NAME, NVARCHAR).label('S_INFO_NAME'),
            C.S_INFO_DELISTDATE,
            C.S_INFO_LISTDATE
        ).outerjoin(B, AINDEXMEMBERSCITICS.S_INFO_WINDCODE == B.S_INFO_WINDCODE).outerjoin(C,
                                                                                           AINDEXMEMBERSCITICS.S_CON_WINDCODE == C.S_INFO_WINDCODE).to_df()
        return df
